/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2017
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var wok = {

    widget: {},

    /**
     * A wrapper of jQuery.ajax function to allow custom bindings.
     *
     * @param settings an extended object to jQuery Ajax settings object
     *   with some extra properties (see below)
     *
     *   resend: if the XHR has failed due to 401, the XHR can be resent
     *     after user being authenticated successfully by setting resend
     *     to true: settings = {resend: true}. It's useful for switching
     *     pages (Guests, Templates, etc.).
     *       e.g., the user wants to list guests by clicking Guests tab,
     *     but he is told not authorized and a login window will pop up.
     *     After login, the Ajax request for /vms will be resent without
     *     user clicking the tab again.
     *       Default to false.
     */
    requestJSON : function(settings) {
        settings['originalError'] = settings['error'];
        settings['error'] = null;
        settings['wok'] = true;
        settings['complete'] = function(req) {
            wok.session.remainingSessionTime = req.getResponseHeader('Session-Expires-On');
            wok.session.remainingSessionTime = (parseInt(wok.session.remainingSessionTime, 10) * 1000);
            if (!wok.session.flagInTimer) {
                wok.session.refreshExpiringCounter();
                wok.session.expiringCounter();
            } else if(wok.session.remainingSessionTime > wok.session.remaingTimeToShowAlert) {
                wok.session.hideExpiringAlert();
                wok.session.refreshExpiringCounter();
                wok.session.flagInTimer = false;
            }
        };
        return $.ajax(settings);
    },

    /**
     * Get the i18 strings.
     */
    getI18n: function(suc, err, url, sync) {
        wok.requestJSON({
            url : url ? url : 'i18n.json',
            type : 'GET',
            resend: true,
            dataType : 'json',
            async : !sync,
            success : suc,
            error: err
        });
    },

    getPeers: function(suc, err) {
        wok.requestJSON({
            url: 'peers',
            type: 'GET',
            contentType: 'application/json',
            dataType: 'json',
            resend: true,
            success: suc,
            error: err ? err : function(data) {
                wok.message.error(data.responseJSON.reason);
            }
        });
    },

    getNotifications: function (suc, err) {
        wok.requestJSON({
            url: 'notifications',
            type : 'GET',
            headers: {'Wok-Robot': 'wok-robot'},
            dataType : 'json',
            success : suc,
            error: err
        });
    },

    removeNotification: function (id) {
        wok.requestJSON({
            url: 'notifications/' + id,
            type : 'DELETE',
            dataType : 'json',
        });
    },

    login : function(settings, suc, err) {
        $.ajax({
            url : "login",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json"
        }).done(suc).fail(err);
    },

    logout : function(suc, err) {
        wok.requestJSON({
            url : 'logout',
            type : 'POST',
            contentType : "application/json",
            dataType : "json"
        }).done(suc).fail(err);
    },

    listPlugins : function(suc, err, sync) {
        wok.requestJSON({
            url : 'config/plugins',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            async : !sync,
            success : suc,
            error : err
        });
    },

    enablePlugin : function(plugin, suc, err) {
        wok.requestJSON({
            url : 'config/plugins/' + encodeURIComponent(plugin) + "/enable",
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    disablePlugin : function(plugin, suc, err) {
        wok.requestJSON({
            url : 'config/plugins/' + encodeURIComponent(plugin) + "/disable",
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    getConfig: function(suc, err, sync) {
        wok.requestJSON({
            url : 'config',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            async : !sync,
            success : suc,
            error : err
        });
    },

    getTasks: function(suc, err) {
        wok.requestJSON({
            url : 'tasks',
            type : 'GET',
            contentType : "application/json",
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getUserLogs : function(suc, err) {
        wok.requestJSON({
            url : 'logs',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err || function(data) {
                wok.message.error(data.responseJSON.reason);
            }
        });
    },

    getFilteredUserLogs : function(suc, err, search) {
        wok.requestJSON({
            url : 'logs?' + search,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err || function(data) {
                wok.message.error(data.responseJSON.reason);
            }
        });
    },

    downloadLogs : function(suc, err, search) {
        wok.requestJSON({
            url : 'logs?'+search+'download=True',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err || function(data) {
                wok.message.error(data.responseJSON.reason);
            }
        });
    }
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('wok.combobox', {
        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.selectDiv.addClass('btn-select dropdown popable');
            this.target.addClass('input');
            this.target.width(this.selectDiv.width()+10);
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function(options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            if (options.length > 0) {
                that.target.after($('<span class="arrow"></span>'));
                that.listControl.on('click', itemTag, function(e) {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if (oldValue !== newValue) {
                        that.target.change();
                    }
                });

                that.selectDiv.click(function(e) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    $.each(items, function(index, item) {
                        that.listControl.append(item);
                    })
                });

                that.target.keyup(function(event) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    var temp = 0;
                    $.each(items, function(index, item) {
                        if (item.text().indexOf(that.target.val()) == 0) {
                            that.listControl.append(item);
                            temp++;
                        }
                    });
                    if (temp > 0 && that.listControl.html() !== '') {
                        that._open();
                    } else {
                        that._close();
                    }
                });
            }
        },

        value : function(value) {
            if (value === undefined) {
                return this.target.val();
            }
            this.target.val(value);
        },

        _dataList : function(options) {
            var item;
            var itemTag = 'li';
            var selectedClass = 'active';
            var items = [];
            var that = this;
            $.each(options, function(index, option) {
                item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                item.data('value', option.value);
                if (option.value === that.target.val()) {
                    item.addClass(selectedClass);
                }
                items.push(item);
            });
            return items;
        },

        clear : function() {
            this.target.val("");
        },

        _open : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (!isOpen) {
                this.selectDiv.addClass('open');
            }
        },

        _close : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (isOpen) {
                this.selectDiv.removeClass('open');
            }
        },

        destroy : function() {
            this.selectDiv.removeClass('btn-select dropdown popable');
            this.target.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * A public function of confirm box.
 *
 * @param msg
 *            type:[object]
 * @param msg.title
 *            The title of the confirm box.
 * @param msg.content
 *            The main text of the confirm box.
 * @param msg.confirm
 *            The text of the confirm button.
 * @param msg.cancel
 *            the text of the cancel button.
 * @param confirmCallback
 *            the callback function of click the confirm button.
 * @param cancelCallback
 *            The callback function of click the cancel and X button.
 * @param closeCallback
 *            The callback function of click the Esc button and blur.
 */
wok.confirm = function(settings, confirmCallback, cancelCallback, closeCallback) {
    "use strict";
    var modalStr = '<div id="wok-confirm-modal" class="modal fade host-modal" tabindex="-1" role="dialog" aria-labelledby="confirmModalLabel" aria-hidden="true"></div>';
    if ($('#wok-confirm-modal ').size() < 1 && $('#modalWindow').size() < 1 ) {
        $(document.body).append(modalStr);
    } else if ($('#wok-confirm-modal ').size() < 1) {
        $('#modalWindow').after(modalStr);
    }

    var confirmboxHeader = [
                '<div class="modal-header', (settings.title === '' || typeof settings.title === 'undefined' ? ' icon' : '' ) ,'">',
                    '<h4 class="modal-title"><i class="fa fa-exclamation-triangle"></i>'+(settings.title || '')+'</h4>',
                '</div>'
    ].join('');

    var confirmboxHtml = [
        '<div class="modal-dialog  modal-sm">',
            '<div class="modal-content">',
                confirmboxHeader,
                '<div class="modal-body">',
                    settings.content,
                '</div>',
                '<div class="modal-footer">',
                    '<button id="button-confirm" class="btn btn-default">' + (settings.confirm || i18n['WOKAPI6004M']) + '</button>',
                    '<button id="button-cancel" class="btn btn-default">' + (settings.cancel || i18n['WOKAPI6003M']) + '</button>',
                '</div>',
            '</div>',
        '</div>'
    ].join('');
    var confirmboxNode = $(confirmboxHtml);
    $('#wok-confirm-modal').append(confirmboxNode);
    $('#wok-confirm-modal').modal('show');
    $('#wok-confirm-modal').on("click", "#button-confirm", function(e) {
        if (confirmCallback) {
            confirmCallback();
        }
        $('#wok-confirm-modal').modal('hide');
    });
    $('#wok-confirm-modal').on("click", "#button-cancel", function(e) {
        if (cancelCallback) {
            cancelCallback();
        }
        $('#wok-confirm-modal').modal('hide');
    });

    $('#wok-confirm-modal').on('hidden.bs.modal', function () {
        close();
    });

    var close = function(){
        "use strict";
        $('#wok-confirm-modal').removeData('bs.modal');
        $('#wok-confirm-modal').remove();
        if (closeCallback) {
            closeCallback();
        }
    };

};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.cookie = {
    set: function(key, value, expireDays) {
        value = encodeURIComponent(value);
        value += '; secure'
        if (expireDays) {
            var expireDate = new Date();
            expireDate.setDate(expireDate.getDate() + expireDays);
            value += '; expires=' + expireDate.toUTCString();
        }
        document.cookie = key + '=' + value;
    },

    get: function(key) {
        var cookieRe = new RegExp(';?\\\s*(' + key + ')=(\s*[^;]*);?', 'g');
        var match = cookieRe.exec(document.cookie);
        return match ? decodeURIComponent(match[2]) : undefined;
    },

    remove: function(key) {
        var utcString = new Date().toUTCString();
        document.cookie = key + '=; expires=' + utcString;
    }
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

$(function(){
$.widget("wok.dataGrid", {
    options: {
        enableSorting: true
    },
    _create: function() {
        var that = this;
        this.element.addClass('wok-datagrid');
        var head = $(this.element.children().get(0));
        var body = $(this.element.children().get(1));
        head.addClass('wok-datagrid-header');
        if(this.options.enableSorting){
            head.children().each(function(){
                var addSorting = "<span>"+$(this).html()+"</span>";
                addSorting += '<span><div class="icon-angle-up sort-up"></div><div class="icon-angle-down sort-down"></div></span>';
                $(this).empty().append(addSorting);
            });
        }
        $('.icon-angle-up', head).click(function(){
            that.sort(head.children().index($(this).parent().parent()), true);
        });
        $('.icon-angle-down', head).click(function(){
            that.sort(head.children().index($(this).parent().parent()), false);
        });
        body.addClass('wok-datagrid-body');
        body.children().addClass('wok-datagrid-row');
        this._setRowBackgroud();
    },
    _setRowBackgroud: function(){
        var i=0, classes=['odd', 'even'];
        $(this.element.children().get(1)).children().each(function(){
            $(this).removeClass('odd');
            $(this).removeClass('even');
            $(this).addClass(classes[i]);
            i = i==0?1:0;
        });
    },
    sort: function(column, assending) {
        var head = $(this.element.children().get(0));
        $('.icon-up-dir', head).removeClass('icon-up-dir').addClass('icon-angle-up');
        $('.icon-down-dir', head).removeClass('icon-down-dir').addClass('icon-angle-down');
        var columnCell = $(head.children().get(column));
        if(assending){
            $('.icon-angle-up', columnCell).removeClass('icon-angle-up').addClass('icon-up-dir');
        }else{
            $('.icon-angle-down', columnCell).removeClass('icon-angle-down').addClass('icon-down-dir');
        }
        var container = $(this.element.children().get(1));
        var nodes = [];
        container.children().each(function(){
            nodes.push($(this));
        });
        nodes.sort(function(a, b){
            aVal = $(a.children().get(column)).attr('val');
            bVal = $(b.children().get(column)).attr('val');
            return aVal.localeCompare(bVal);
        });
        if(!assending) nodes.reverse();
        container.empty();
        for(var i=0;i<nodes.length;i++){
            container.append(nodes[i]);
        }
        this._setRowBackgroud();
    },
    filter: function(keyword) {
        keyword = keyword.toLowerCase();
        var container = $(this.element.children().get(1));
        container.children().each(function(){
            var hide = true;
            $(this).children().each(function(){
                if($(this).attr('val')&&$(this).attr('val').toLowerCase().indexOf(keyword)!=-1){
                    hide = false;
                    return false;
                }
            });
            $(this).css('display', hide?'none':'');
        });
        this._setRowBackgroud();
    },
    addRow: function(rowNode){
        $(rowNode).addClass('wok-datagrid-row');
        this._setRowBackgroud();
    },
    deleteRow: function(rowNode){
        $(rowNode).remove();
        this._setRowBackgroud();
    },
    _destroy: function() {
        this.element.removeClass('wok-datagrid');
        var head = $(this.element.children().get(0));
        var body = $(this.element.children().get(1));
        head.removeClass('wok-datagrid-header');
        if(this.options.enableSorting){
            head.children().each(function(){
                var oriContent = $($(this).children().get(0)).html()
                $(this).empty().append(oriContent);
            });
        }
        body.removeClass('wok-datagrid-body');
        body.children().removeClass('wok-datagrid-row odd even');
    }
});
});
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// Sets defalt Datatable options for Wok

wok.initCompleteDataTableCallback = function(e) {
    var tableId = $('#'+e.sTableId+'_wrapper');
    $('.dataTables_filter input',tableId).attr("placeholder", "Filter");
    $(".dataTables_filter label",tableId).contents().filter(function() {
    return this.nodeType === 3;
    }).wrap('<div class="sr-only" />');
    $(".dataTables_length label",tableId).contents().filter(function() {
    return this.nodeType === 3;
    }).wrap('<div class="sr-only" />');
    $(".dataTables_length select",tableId).selectpicker();
    $(".dataTables tfoot",tableId).remove();
};

$.extend( true, $.fn.dataTable.defaults, {
    "sPaginationType": "full_numbers",
    "dom": '<"row"<"col-sm-12 filter"<"pull-right"l><"pull-right"f>>><"row"<"col-sm-12"t>><"row datatable-footer"<"col-sm-6 pages"p><"col-sm-6 info"i>>',
    "initComplete": function(settings, json) {
        wok.initCompleteDataTableCallback(settings);
    }
});/*
 * Project Wok
 *
 * Copyright IBM Corp, 2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('wok.filterselect', {
        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.selectDiv.addClass('btn-select dropdown popable');
            this.target.addClass('input');
            this.target.width(this.selectDiv.width()+10);
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function(options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            if (options.length > 0) {
                that.target.after($('<span class="arrow"></span>'));
                that.listControl.on('click', itemTag, function(e) {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if (oldValue !== newValue) {
                        that.target.change();
                        that.target.removeClass("invalid-field");
                    }
                });

                that.selectDiv.click(function(e) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    if (items.length === 0) {
                        that.listControl.html(i18n['WOKAPI6005E']);
                    } else {
                        $.each(items, function(index, item) {
                            that.listControl.append(item);
                        });
                    }
                });

                that.target.keyup(function(event) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    var temp = 0;
                    $.each(items, function(index, item) {
                        if (item.text().indexOf(that.target.val()) == 0) {
                            that.listControl.append(item);
                            temp++;
                        }
                    });
                    if (that.listControl.html() === '') {
                        that.target.addClass("invalid-field");
                        that.listControl.html(i18n['WOKAPI6005E']);
                    } else {
                        that.target.removeClass("invalid-field");
                    }
                    if (temp > 0) {
                        that._open();
                    }
                });
            }
        },

        value : function(value) {
            if (value === undefined) {
                return this.target.val();
            }
            this.target.val(value);
        },

        _dataList : function(options) {
            var item;
            var itemTag = 'li';
            var selectedClass = 'active';
            var items = [];
            var that = this;
            $.each(options, function(index, option) {
                item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                item.data('value', option.value);
                if (option.value === that.target.val()) {
                    item.addClass(selectedClass);
                }
                items.push(item);
            });
            return items;
        },

        clear : function() {
            this.target.val("");
        },

        _open : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (!isOpen) {
                this.selectDiv.addClass('open');
            }
        },

        _close : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (isOpen) {
                this.selectDiv.removeClass('open');
            }
        },

        destroy : function() {
            this.selectDiv.removeClass('btn-select dropdown popable');
            this.target.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover')
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function($) {
    $.fn.serializeObject = function() {
        var formDataArray = $(this).serializeArray();
        var formData = new Object();
        $.each(formDataArray, function(index, data) {
            formData.setDeepValue(data.name, data.value);
        });
        return formData;
    };
}(jQuery));

(function($) {
    $.fn.fillWithObject = function(obj) {
        $(this).find("input").each(function(){
            switch($(this).attr('type')) {
                case 'text':
                case 'number':
                    $(this).val(obj.getDeepValue($(this).attr("name")));
                    break;
                case 'radio':
                case 'checkbox':
                    var a=String($(this).val());
                    var b=String(obj.getDeepValue($(this).attr("name")));
                    $(this).prop("checked",(a==b));
                    break;
                default:
                    break;
                }
            });
     };
}(jQuery));
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.widget.Grid = function(opts) {
    "use strict";
    this.opts = $.extend({}, this.opts, opts);
    this.createDOM();
    this.reload();
};

wok.widget.Grid.prototype = (function() {
    "use strict";
    var htmlStr = [
        '<div id="{id}" class="grid wok-grid">',
            '<div class="wok-grid-message hidden">',
                '<div class="alert alert-danger fade in" role="alert">',
                    '<p><strong>{message}</strong> ',
                    '<span class="detailed-text"></span></p>',
                    '<p><button class="btn btn-primary btn-xs retry-button">',
                        '{buttonLabel}',
                    '</button></p>',
                '</div>',
            '</div>',
            '<div class="grid-content wok-grid-content">',
                    '<table class="wok-table table table-striped">',
                        '<thead class="wok-grid-header-container"></thead>',
                        '<tbody class="wok-grid-body-container">',
                        '</tbody>',
                    '</table>',
            '</div>',
            '<div class="wok-grid-mask hidden">',
                '<div class="wok-grid-loader-container">',
                    '<div class="wok-grid-loading">',
                        '<div class="wok-grid-loading-icon"></div>',
                        '<div class="wok-grid-loading-text">',
                            '{loading}',
                        '</div>',
                    '</div>',
                '</div>',
            '</div>',
        '</div>'
    ].join('');

    var setupHeaders = function(header, body, fields) {
        var colGroup = $('<colgroup></colgroup>').appendTo(header);
        var headerRow = $('<tr></tr>').appendTo(header);
        $.each(fields || [], function(i, field) {
            $('<col class="' +
                field['class'] +
            '"/>')
                .appendTo(colGroup);
            $('<th><div class="wok-text-header">' +
                field['label'] +
            '</div></th>').appendTo(headerRow);
        });

        var totalWidth = 0;
        return totalWidth;
    };

    var getValue = function(name, obj) {
        var result;
        if(!Array.isArray(name)) {
            name=name.parseKey();
        }
        if(name.length!==0) {
            var tmpName=name.shift();
            if(obj[tmpName]!==undefined) {
                    result=obj[tmpName];
            }
            if(name.length!==0) {
                    result=getValue(name,obj[tmpName]);
            }
        }
        return(result);
    };

    var fillBody = function(container, fields) {
        var data = this.data;
        $.each(data, function(i, row) {
            var rowNode = $('<tr></tr>').appendTo(container);
            $.each(fields, function(fi, field) {
                var value = getValue(field['name'], row);
                $('<td><div class="wok-cell-text"' + (field['makeTitle'] === true ? ' title="' + value + '"' : '' ) + '>' + value.toString() + '</div></td>'
                ).appendTo(rowNode);
            });
        });
    };

    var stylingRow = function(row, className, add) {
        var index = $(row).index() + 1;
        $('tr', this.bodyContainer)
            .removeClass(className);

        if(add === false) {
            return;
        }

        $('tr:nth-child(' + index + ')', this.bodyContainer)
            .addClass(className);
    };

    var setBodyListeners = function() {
        if(this['opts']['rowSelection'] !== 'disabled') {
            $('tr', this.gridBody).on('mouseover', {
                grid: this
            }, function(event) {
                if (! $(this).hasClass('no-hover')) {
                    stylingRow.call(event.data.grid, this, 'hover');
                }
            });

            $('tr', this.gridBody).on('mouseout', {
                grid: this
            }, function(event) {
                stylingRow.call(event.data.grid, this, 'hover', false);
            });

            $('tr', this.gridBody).on('click', {
                grid: this
            }, function(event) {
                var grid = event.data.grid;
                grid.selectedIndex = $(this).index();
                stylingRow.call(grid, this, 'selected');
                grid['opts']['onRowSelected'] && grid['opts']['onRowSelected']();
            });
        }

    };

    var setData = function(data) {
        this.data = data;
        fillBody.call(this, this.bodyContainer, this['opts']['fields']);
        setBodyListeners.call(this);
    };

    var getSelected = function() {
        return this.selectedIndex >= 0 ? this.data[this.selectedIndex] : null;
    };

    var showMessage = function(msg) {
        $('.detailed-text', this.messageNode).text(msg);
        $(this.messageNode).removeClass('hidden');
    };

    var hideMessage = function() {
        $(this.messageNode).addClass('hidden');
    };

    var reload = function() {
        var data = this['opts']['data'];
        if(!data) {
            return;
        }

        $(this.messageNode).addClass('hidden');

        if($.isArray(data)) {
            return this.setData(data);
        }

        if($.isFunction(data)) {
            var loadData = data;
            $(this.maskNode).removeClass('hidden');
            loadData($.proxy(function(data) {
                this.setData(data);
                $(this.maskNode).addClass('hidden');
            }, this));
        }
    };

    var createDOM = function() {
        var containerID = this['opts']['container'];
        var container = $('#' + containerID);
        var gridID = this['opts']['id'];
        var rowSelection = this['opts']['rowSelection'] || 'single';
        var domNode = $(wok.substitute(htmlStr, {
            id: gridID,
            loading: i18n['WOKGRD6001M'],
            message: i18n['WOKGRD6002M'],
            buttonLabel: i18n['WOKGRD6003M'],
            detailedLabel: i18n['WOKGRD6004M']
        })).appendTo(container);
        this.domNode = domNode;

        var theTable = $('.wok-table', domNode);
        var theContainer = $('.wok-grid-content', domNode);
        var headerContainer = $('.wok-grid-header-container', domNode);
        var bodyContainer = $('.wok-grid-body-container', domNode);
        setupHeaders(headerContainer, bodyContainer, this['opts']['fields']);
        this.theTable = theTable;
        this.theContainer = theContainer;
        this.headerContainer = headerContainer;
        this.bodyContainer = bodyContainer;

        var height = domNode.height();
        var width = domNode.width();

        var title = this['opts']['title'];
        var titleNode = null;
        if(title) {
            titleNode = $('<caption class="sr-only">' + title + '</caption>').prependTo(theTable);
        }

        var toolbarButtons = this['opts']['toolbarButtons'];
        var toolbarNode = null;
        var btnHTML, dropHTML = [];
        if(toolbarButtons) {
            toolbarNode = $('<div class="btn-group"></div>');
            toolbarNode.prependTo(theContainer);
            if(toolbarButtons.length > 1) {
                dropHTML = ['<div class="dropdown menu-flat">',
                    '<button id="wok-dropdown-button-', containerID, '" class="btn btn-primary dropdown-toggle" type="button" data-toggle="dropdown">',
                    '<span class="edit-alt"></span>Actions<span class="caret"></span>',
                    '</button>',
                    '<ul class="dropdown-menu"></ul>',
                    '</div>'
                ].join('');
                $(dropHTML).appendTo(toolbarNode);
                $.each(toolbarButtons, function(i, button) {
                    btnHTML = [
                        '<li role="presentation"', button.critical === true ? ' class="critical"' : '', '><a data-toggle="modal"',
                        button.id ? (' id="' + button.id + '"') : '',
                        ' class="', button.disabled === true ? ' disabled' : '', '">',
                        button.class ? ('<i class="' + button.class) + '"></i>' : '',
                        button.label,
                        '</a></li>'
                    ].join('');
                    var btnNode = $(btnHTML).appendTo(toolbarNode[0].children[0].children[1]);
                    button.onClick && btnNode.on('click', button.onClick);
                });
            }else {
                $.each(toolbarButtons, function(i, button) {
                    btnHTML = [
                        '<button data-dismiss="modal" ',
                            button['id'] ? (' id="' + button['id'] + '"') : '',
                            ' class="btn btn-primary',
                                button['class'] ? (' ' + button['class']) : '',
                                '"',
                                button['disabled'] === true ? ' disabled' : '',
                                '>',
                                button['label'],
                        '</button>'
                    ].join('');
                    var btnNode = $(btnHTML).appendTo(toolbarNode);
                    button['onClick'] &&
                        btnNode.on('click', button['onClick']);
                });
            }

        }

        // var domHeight = domNode && $(domNode).height() || 0;
        // var toolbarHeight = toolbarNode && $(toolbarNode).height() || 0;
        // var maskHeight = domHeight - toolbarHeight;

        // var maskContainer = $('.wok-grid-loader-container',domNode);
        // maskContainer.css({'top': toolbarHeight+'px', 'height': maskHeight+'px'});
        // this.maskContainer = maskContainer;

        var maskNode = $('.wok-grid-mask', domNode);
        this.maskNode = maskNode;

        var messageNode = $('.wok-grid-message', domNode);
        this.messageNode = messageNode;

        //fixTableLayout.call(this);

        var gridBody = $('.wok-grid-body', domNode);
        this.gridBody = gridBody;

        var data = this['opts']['data'];

        $('.retry-button', domNode).on('click', {
            grid: this
        }, function(event) {
            event.data.grid.reload();
        });
    };

    return {
        opts: {
            container: null,
            id: null,
            rowSelection: 'single',
            onRowSelected: null,
            title: null,
            toolbarButtons: null,
            fields: null
        },
        createDOM: createDOM,
        setData: setData,
        getSelected: getSelected,
        reload: reload,
        //destroy: destroy,
        showMessage: showMessage
    };
})();
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.lang = {
    all: function() {
        return {
          'en_US': 'English (US)',
          'zh_CN': '中文（简体）',
          'pt_BR': 'Português (Brasil)',
          'de_DE': 'Deutsch (Deutschland)',
          'es_ES': 'Español (España)',
          'fr_FR': 'Français (France)',
          'it_IT': 'Italiano (Italia)',
          'ja_JP': '日本語 (日本)',
          'ko_KR': '한국어 (대한민국)',
          'ru_RU': 'Русский (Россия)',
          'zh_TW': '中文（繁體）'
        };
    },

    /**
     * Language is determined by the following sequence:
     * 1) Cookie setting; or if not set ->
     * 2) HTML DOM lang attribute; or if not set ->
     * 3) DEFAULT (en_US).
     */
    get: function() {
        return wok.cookie.get('wokLang') ||
            $('html').prop('lang') ||
            'en_US';
    },

    /**
     * Locale is determined by the following sequence:
     * 1) Cookie setting; or if not set ->
     * 2) HTML DOM lang attribute; or if not set ->
     * 3) DEFAULT (en_US).
     */
    get_locale: function() {
        return wok.cookie.get('wokLocale') ||
            $('html').prop('lang').replace('_', '-') ||
            'en-US';
    },

    set_locale: function(selected_locale) {
        wok.cookie.set('wokLocale', selected_locale, 365);
    },

    set: function(lang) {
        wok.cookie.set('wokLang', lang, 365);
    }
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * new wok.widget.LineChart({
 *   node: 'line-chart-cpu',
 *   id: 'line-chart',
 *   type: 'value'
 * });
 */


wok.widget.LineChart = function(params) {
    var container = $('#' + params['node']);
    container.addClass('chart-container');
    var height = container.height();
    var width = container.width();
    var numHLines = 4;
    var linesSpace = height / numHLines;
    var period = params['period'] || 20;
    var xFactor = width / period;
    var yFactor = height / 100;
    var xStart = params['xStart'] || 0;
    var linesOffset = 0;
    var canvasID = params['id'];
    var maxValue = params['maxValue'] || -Infinity;
    var type = params['type'];
    var chartVAxis = null;
    var chartTitle = null;
    var chartLegend = null;
    var seriesMap = {};
    var formatSettings = {};

    var setMaxValue = function(newValue) {
        maxValue = newValue;
    };

    /**
     *
     * settings: {
     *   'class': 'disk-read-rate'
     * }
     */
    var updateUI = function(data) {
        var container = $('#' + params['node']);
        if(!container.length) {
            return;
        }

        if(!$.isArray(data)) {
            data = [data];
        }
        var seriesCount = 0;
        var singleSeries = data.length === 1;
        var firstSeries = data[0];
        var converters = params['converters'] ? params['converters'] : '';

        // TODO: Multiple axes support.
        if(type === 'value') {
            $.each(data, function(i, series) {
                if(series['max'] > maxValue) {
                    maxValue = series['max'];
                    // Intoduce converter to format data as per converter's to function
                    var converter = series['converter'];
                    if(converter){
                        converter = converters[converter] ? converters[converter] : '';
                    }
                    formatSettings = {
                        base: series['base'],
                        unit: series['unit'],
                        fixed: series['fixed'],
                        converter: converter
                    };
                }
            });
        }


        var defs = [
            '<defs>',
                '<pattern id="patternbg" x="0" y="0" width="6" height="6" patternUnits="userSpaceOnUse">',
                    '<rect x="0" y="0" width="3" height="6" style="stroke:none; fill: #eeeeee;"></rect>',
                '</pattern>',
            '</defs>'
        ].join('');

        var canvasNode = $('#' + canvasID);
        canvasNode.length && canvasNode.remove();
        var htmlStr = [
          '<svg id="', canvasID, '" class="line-chart"',
              ' height="', height, '" width="', width, '"',
          '>',
            defs,
            '<rect height="', height, '" width="', width, '" class="background" />'
        ];


        var maxValueLabel = i18n['WOKHOST6001M'] + ' ' +
            (type === 'value'
                ? wok.formatMeasurement(maxValue, formatSettings)
                : '100%');
        if(!chartVAxis) {
            chartVAxis = $('<div class="chart-vaxis-container">' +
                maxValueLabel +
                '</div>'
            );
            container.before(chartVAxis);
        }
        else {
            chartVAxis.text(maxValueLabel);
        }


        seriesNames = [];
        $.each(data, function(i, series) {
            var points = series['points'];
            var className = series['class'];
            var latestPoint = points.slice(-1).pop();
            xStart = latestPoint['x'] - period;
            htmlStr.push('<path',
                ' class="series', className ? ' ' + className : '', '"',
                ' d="M 0,92 '
            );
            var first = true;
            $.each(points, function(i, point) {
                if(first) {
                    first = false;
                }
                else {
                    htmlStr.push(' ');
                }

                var x = xFactor * (point['x'] - xStart);
                var y = height - yFactor * (type === 'value' ?
                    point['y'] * 100 / maxValue :
                    point['y']
                );
                htmlStr.push(x, ',', y);
            });
            htmlStr.push(' 310,92z" />');

            htmlStr.push('<polyline',
                ' class="series', className ? ' ' + className : '', '"',
                ' points="'
            );
            var first = true;
            $.each(points, function(i, point) {
                if(first) {
                    first = false;
                }
                else {
                    htmlStr.push(' ');
                }

                var x = xFactor * (point['x'] - xStart);
                var y = height - yFactor * (type === 'value' ?
                    point['y'] * 100 / maxValue :
                    point['y']
                );
                htmlStr.push(x, ',', y);
            });
            htmlStr.push('" />');

        });

        htmlStr.push('</svg>');

        var canvasNode = $(htmlStr.join('')).appendTo(container);

        if(!chartLegend) {
            chartLegend = $('<div class="chart-legend-container"></div>');
            container.before(chartLegend);
        }
        else {
            chartLegend.empty();
        }
        $('polyline.series', canvasNode).each(function(i, polyline) {
            var wrapper = $('<div class="legend-wrapper"></div>')
                .appendTo(chartLegend);
            $([
                '<svg class="legend-icon" width="5" height="40">',
                    '<rect  width="5" height="40" />',
                '</svg>'
            ].join('')).appendTo(wrapper);
            $('rect', wrapper).css({
                fill: $(polyline).css('stroke')
            });
            var label = data[i]['legend'];
            var base = data[i]['base'];
            var latestPoint = data[i]['points'].slice(-1).pop();
            var latestValue = latestPoint['y'];

            if(type === 'value') {
                latestValue = wok.formatMeasurement(
                    latestValue,
                    formatSettings
                );
            }
            else {
                // Format value if converter is passed.
                converter = data[i]['converter'];
                var converter_to = ''
                if(converter){
                    converter_to = converters[converter] ? converters[converter]['to'] : '';
                }

                latestValue = { v: converter_to ? converter_to(latestValue) : latestValue, s : '%' };
            }
            $('<div class="latest-value"><span class="number">' + latestValue.v + '</span></div>').appendTo(wrapper);
            $('<span class="legend-label">'+ latestValue.s +'</span><span class="legend-string">'+ label + '</span>').appendTo(wrapper[0].children[1]);

        });
    };

    return {
        setMaxValue: setMaxValue,
        updateUI: updateUI
    }
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.widget.List = function(opts) {
    "use strict";
    this.opts = $.extend({}, this.opts, opts);
    this.createDOM();
    this.reload();
};

wok.widget.List.prototype = (function() {
    "use strict";

    var htmlStr = [
        '<div id="{id}-section" class="panel panel-default">',
            '<div class="panel-heading">',
            '</div>',
            '<div id="content-{id}" class="panel-body">',
                '<div id="{id}-container">',
                    '<div class="wok-list-message clearfix hidden">',
                        '<div class="alert alert-danger fade in" role="alert">',
                            '<p><strong>{message}</strong> ',
                            '<span class="detailed-text"></span></p>',
                            '<p><button class="btn btn-primary btn-xs retry-button">',
                                '{buttonLabel}',
                            '</button></p>',
                        '</div>',
                    '</div>',
                    '<div id="{id}-action-group" class="wok-list-action-button-container">',
                        '<div class="dropdown mobile-action">',
                            '<button class="btn btn-primary dropdown-toggle" type="button" data-toggle="dropdown" aria-expanded="false">',
                                '<span class="mobile-action-label">{actions}</span>',
                            '</button>',
                            '<ul class="dropdown-menu" role="menu">',
                            '</ul>',
                        '</div>',
                    '</div>',
                    '<div id="{id}" class="row clearfix">',
                        '<div class="wok-list-content">',
                            '<div class="wok-list" id="list">',
                            '</div>',
                        '</div>',
                    '</div>',
                    '<div class="wok-list-mask hidden">',
                        '<div class="wok-list-loader-container">',
                            '<div class="wok-list-loading">',
                                '<div class="wok-list-loading-icon"></div>',
                                '<div class="wok-list-loading-text">',
                                    '{loading}',
                                '</div>',
                            '</div>',
                        '</div>',
                    '</div>',
                '</div>',
            '</div>',
        '</div>'
    ].join('');

    var getValue = function(name, obj) {
        var result;
        if (!Array.isArray(name)) {
            name = name.parseKey();
        }
        if (name.length !== 0) {
            var tmpName = name.shift();
            if (obj[tmpName] !== undefined) {
                result = obj[tmpName];
            }
            if (name.length !== 0) {
                result = getValue(name, obj[tmpName]);
            }
        }
        return (result);
    };

    var fillButtons = function(btnContainer){
        var toolbarButtons = this.opts.toolbarButtons;
        $.each(toolbarButtons, function(i, button) {
                var btnHTML = [
                    '<li class="',
                    (button.critical === true ? 'critical' : ''),
                    ,'">',
                        '<btn data-dismiss="modal"',
                        (button.id ? (' id="' + button.id + '"') : ''),
                        ' class="btn btn-primary"',
                        (button.disabled === true ? ' disabled="disabled"' : ''),
                        '">',
                            button.class ? ('<i class="' + button.class) + '"></i> ' : ' ',
                            button.label,
                        '</btn>',
                    '</li>'
                ].join('');
                var btnNode = $(btnHTML).appendTo(btnContainer);
                button.onClick && btnNode.on('click', button.onClick);
        });
    };

    var fillBody = function(container, fields) {
        var data = this.data;
        var converters = this.opts.converters;
        var tbody = ($('ul', container).length && $('ul', container)) || $('<ul></ul>').appendTo(container);
        tbody.empty();
        if (typeof data !== 'undefined' && data.length > 0) {
            $.each(data, function(i, row) {
                var rowNode = $('<li></li>').appendTo(tbody);
                var columnNodeHTML;
                var columnData = '';
                var state = '';
                var styleClass = '';
                var checkboxName = $('ul', container).parent().parent().parent().attr('id') + '-check' || $(container).parent().parent().parent().attr('id') + '-check';
                $.each(fields, function(fi, field) {
                    var value = getValue(field.name, row);
                    if(field.converter){
                        var converter = field.converter;
                        if(converters[converter]){
                            var to = converters[converter]['to'];
                            value = to(value);
                        }else{
                            console.error('converter ' + converters[converter] + ' not defined');
                        }
                    }
                    if (field.type === 'status' && field.name === 'enabled') {
                        styleClass = (value === true ? '' : ' disabled');
                        state = [
                            '<span class="wok-list-item-status ',
                            value === true ? 'enabled' : 'disabled',
                            '"><i class="fa fa-power-off"></i><span class="sr-only">',
                            value === true ? 'Enabled' : 'Disabled',
                            '</span></span>'
                        ].join('');
                    }
                    columnData += (field.type === 'name') ? ('<span role="status" class="wok-list-loading-icon-inline"></span><span class="wok-list-name '+field.cssClass+'" title="'+field.label+'">'+value.toString()+'</span>') : (field.type !== 'status' ? '<span class="wok-list-description '+field.cssClass+'" title="'+field.label+'">' + value.toString() + '</span>' : '');
                    columnNodeHTML = [
                        '<input class="wok-checkbox" type="checkbox" name="'+checkboxName+'" id="wok-list-',i+1,'-check" />',
                            '<label for="wok-list-',i+1,'-check" class="wok-list-cell', styleClass, '">',
                                state,
                                columnData,
                            '</div>',
                        '</label>'
                    ].join('');
                });
                $(columnNodeHTML).appendTo(rowNode);
            });
        }
    };

    var stylingRow = function(grid, className) {
        $('li',grid.bodyContainer).removeClass(className);
        $.each(grid.selectedIndex, function(){
            var nth = this + 1;
            $('li:nth-child('+nth+')',grid.bodyContainer).addClass(className);
        });
    };

    var setBodyListeners = function() {
        if (this['opts']['rowSelection'] !== 'disabled') {
            $('li:not(.generating) input[type="checkbox"]', this.bodyContainer).on('change', {
                grid: this
            },function(event) {
                var grid = event.data.grid;
                grid.selectedIndex = [];
                $("li > :checkbox:checked", this.bodyContainer).map(function() {
                    return $(this).parent().index();
                }).each(function() {
                    grid.selectedIndex.push(this);
                });
                if ($('.mobile-action-count',grid.buttonActionContainer).length) {
                    $('.mobile-action-count',grid.buttonActionContainer).remove();
                }
                if(grid.selectedIndex.length){
                    $(grid.buttonActionContainer).append('<span class="mobile-action-count"> ( <strong>'+grid.selectedIndex.length+' item(s)</strong> selected )</span>');
                }
                stylingRow.call(grid, grid, 'selected');
                grid['opts']['onRowSelected'] && grid['opts']['onRowSelected']();
            });
        }
    };

    var setData = function(data) {
        this.data = data;
        fillBody.call(this, this.bodyContainer, this.opts.fields);
        setBodyListeners.call(this);
    };

    var getSelected = function() {
        var selectedItems = [];
        for (var i = 0; i < this.selectedIndex.length; i++ ){
            var value = this.selectedIndex[i];
            selectedItems.push(this.data[value]);
        }
        // return this.selectedIndex >= 0 ? this.data[this.selectedIndex] : null;
        return selectedItems;
    };

    var showMessage = function(msg) {
        $('.detailed-text', this.messageNode).text(msg);
        $(this.messageNode).removeClass('hidden');
    };

    var hideMessage = function() {
        $(this.messageNode).addClass('hidden');
    };

    var reload = function() {
        var data = this.opts.data;
        if (!data) {
            return;
        }

        if ($.isArray(data)) {
            return this.setData(data);
        }

        if ($.isFunction(data)) {
            var loadData = data;
            $(this.maskNode).removeClass('hidden');
            loadData($.proxy(function(data) {
                this.setData(data);
                $(this.maskNode).addClass('hidden');
            }, this));
        }
    };

    var createDOM = function() {
        var containerID = this.opts.container;
        var container = $('#' + containerID);
        var gridID = this.opts.id;
        var data = this.opts.data;
        var rowSelection = this.opts.rowSelection || 'single';
        var domNode = $(wok.substitute(htmlStr, {
            id: gridID,
            loading: i18n.WOKGRD6001M,
            message: i18n.WOKGRD6002M,
            buttonLabel: i18n.WOKGRD6003M,
            detailedLabel: i18n.WOKGRD6004M,
            actions: i18n.WOKSETT0012M
        })).appendTo(container);
        this.domNode = domNode;


        var titleContainer = $('.panel-heading', domNode);
        this.titleContainer = titleContainer;

        var title = this.opts.title;
        var titleNode = null;

        if (title) {
            titleNode = $('<h3 class="panel-title">' + title + '</h3>').appendTo(titleContainer);
        }

        var bodyContainer = $('.wok-list', domNode);
        this.bodyContainer = bodyContainer;

        var selectButtonContainer = $('.wok-list-action-button-container', domNode);
        this.selectButtonContainer = selectButtonContainer;

        var buttonActionGroupContainer = $('.wok-list-action-button-container .dropdown-menu', domNode);
        this.buttonActionGroupContainer = buttonActionGroupContainer;

        var buttonActionContainer = $('.mobile-action .dropdown-toggle.btn', domNode);
        this.buttonActionContainer = buttonActionContainer;

        var gridBody = $('.wok-list-content', domNode);
        this.gridBody = gridBody;

        var maskNode = $('.wok-list-mask', domNode);
        this.maskNode = maskNode;

        var messageNode = $('.wok-list-message', domNode);
        this.messageNode = messageNode;

        fillButtons.call(this,this.buttonActionGroupContainer);

        $('.retry-button', domNode).on('click', {
            grid: this
        }, function(event) {
            event.data.grid.reload();
        });

    };

    return {
        opts: {
            container: null,
            id: null,
            rowSelection: 'single',
            onRowSelected: null,
            title: null,
            toolbarButtons: null,
            frozenFields: null,
            fields: null
        },
        createDOM: createDOM,
        setData: setData,
        getSelected: getSelected,
        reload: reload,
        showMessage: showMessage
    };
})();
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2017
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.login_main = function() {
    "use strict";
    var i18n;
    wok.getI18n(function(i18nObj){
        i18n = i18nObj;
     }, false, "i18n.json", true);

    // verify if language is available
    var selectedLanguage = wok.lang.get();
    if ($('#userLang option[value="'+selectedLanguage+'"]').length == 1)  {
        $('#userLang').val(selectedLanguage);
        $('#userLang option[value="'+selectedLanguage+'"]').attr("selected", "selected");
        $('.filter-option:first').parent().attr('title',$('#userLang option[value="'+selectedLanguage+'"]').text());
        $('.filter-option:first').text($('#userLang option[value="'+selectedLanguage+'"]').text());
    }

    // verify if locale is available
    var selectedLocale = wok.lang.get_locale();
    if ($('#userLocale option[value="'+selectedLocale+'"]').length == 1) {
        $('#userLocale').val(selectedLocale);
        $('#userLocale option[value="'+selectedLocale+'"]').attr("selected", "selected");
        $('.filter-option:last').parent().attr('title',$('#userLocale option[value="'+selectedLocale+'"]').text());
        $('.filter-option:last').text($('#userLocale option[value="'+selectedLocale+'"]').text());
    }

    $('#userLang').on('change', function() {
        wok.lang.set($(this).val());
        location.reload();
    });

    $('#userLocale').on('change', function() {
        wok.lang.set_locale($(this).val());
    });

    var query = window.location.search;
    var error = /.*error=(.*?)(&|$)/g.exec(query);
    if (error && error[1] === "sessionTimeout") {
        $("#errorArea").html(i18n["WOKAUT0001E"]);
        $("#errorArea").show();
    }

    var userNameBox = $('#username');
    var passwordBox = $('#password');
    var loginButton = $('#btn-login');

    var login = function(event) {
        $("#login").hide();
        $("#logging").show();

        var userName = userNameBox.val();
        userName && wok.user.setUserName(userName);
        var settings = {
            username: userName,
            password: passwordBox.val()
        };

        wok.login(settings, function(data) {
            var query = window.location.search;
            var next  = /.*next=(.*?)(&|$)/g.exec(query);
            if (next) {
                var next_url = decodeURIComponent(next[1]);
            }
            else {
                var lastPage = wok.cookie.get('lastPage');
                var next_url = lastPage ? lastPage.replace(/\"/g,'') : "/";
            }
            wok.cookie.set('user_role', data.role);
            window.location.replace(window.location.pathname.replace(/\/+login.html/, '') + next_url);
        }, function(jqXHR, textStatus, errorThrown) {
            if (jqXHR.responseText == "") {
                $("#errorArea").html(i18n["WOKAUT0002E"]);
                $("#errorArea").show();
            } else if ((jqXHR.responseJSON != undefined) &&
                       ! (jqXHR.responseJSON["reason"] == undefined)) {
                $("#errorArea").html(jqXHR.responseJSON["reason"]);
                $("#errorArea").show();
            }
            $("#logging").hide();
            $("#login").show();
        });

        return false;
    };

    wok.logos('#wok-logos',true);
    $('#btn-login').bind('click', login);
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2016-2017
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


wok.plugins = undefined;
wok.listPlugins(function(result) {
    wok.plugins = result;
}, function(data) {
    wok.plugins = [];
    wok.message.error(data.responseJSON.reason);
});

wok.logos = function(element, powered) {
    powered = (typeof powered === 'undefined') ? false : true;
    var genLogos  = function(obj){
        var objHtml = [];
        var plugins = [];
        for (var i in obj) {
            plugins.push(obj[i])
        }
        plugins.sort(function(a, b) {
            return a.name.localeCompare( b.name );
        });
        $(plugins).each(function(i, plugin) {
            var name = plugin.name[0].toUpperCase() + plugin.name.slice(1);
            if(plugin.image && plugin.version) {
                objHtml.push(
                    '<li>',
                        '<img src="',plugin.image,'" longdesc="',name,' logotype" alt="',name,' plugin - ',plugin.version,'" title="',name,' plugin - ',plugin.version,'" />',
                        '<span class="plugin-title">',name,'</span>',
                        '<span class="plugin-version">',plugin.version,'</span>',
                    '</li>'
                );
            }else if(plugin.image) {
                objHtml.push(
                    '<li>',
                        '<img src="',plugin.image,'" longdesc="',name,' logotype" alt="',name,' plugin" title="',name,' plugin" />',
                        '<span class="plugin-title">',name,'</span>',
                    '</li>'
                );
            }
        });
        return objHtml.join('');
    };

    var retrieveVersion = function(url) {
        var version = "";
        $.ajax({
            dataType: "json",
            url : url+'/config',
            async : false,
            success : function(data) {
                version = data.version;
            },
        });
        return version;
    };

    var checkImage = function (testurl) {
     var http = $.ajax({
        type:"HEAD",
        url: testurl,
        async: false
      })
      return http.status;
    }

    var pluginUrl = 'plugins/{plugin}';
    var buildLogos = function() {
        // Make wok.plugins is ready to be used
        if (wok.plugins == undefined) {
            setTimeout(buildLogos, 2000);
            return;
        }

        var logos = [];
        var  obj = {};
        if(wok.plugins && wok.plugins.length > 0) {
            $(wok.plugins).each(function(i, p) {
                if (p.enabled === false) {
                    return true;
                }
                var url = wok.substitute(pluginUrl, {
                    plugin: p.name
                });
                obj[i] = {
                    name : p.name
                }
                var pluginVersions;
                pluginVersions = retrieveVersion(url);
                if(pluginVersions && pluginVersions.length > 0){
                    obj[i].version = pluginVersions;
                }
                var imagepath = url+'/images/'+p.name;
                if(checkImage(imagepath+'.svg') == 200) {
                    obj[i].image = wok.plugins[i].image = imagepath+'.svg';
                }
                else if(checkImage(imagepath+'.png') == 200) {
                    obj[i].image = wok.plugins[i].image =  imagepath+'.png';
                }
            });
            var generatedLogos = genLogos(obj);
            if(generatedLogos.length > 0) {
                $(element).append(generatedLogos);
                if(powered) {
                    $(element).parentsUntil('.container').find('.powered').removeClass('hidden');
                }else {
                    $(element).parentsUntil('.container').find('.powered').remove();
                }
            }
        }
    };
    buildLogos();

 };
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2017
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

wok.NOTIFICATION_INTERVAL = 2000
wok.tabMode = {};
wok.pluginsColor = {};

wok.config = undefined;
wok.getConfig(function(result) {
    wok.config = result;
}, function() {
    wok.config = {};
});

wok.notificationListeners = {};
wok.addNotificationListener = function(msg, func) {
    var listenerArray = wok.notificationListeners[msg];
    if (listenerArray == undefined) {
        listenerArray = [];
    }
    listenerArray.push(func);
    wok.notificationListeners[msg] = listenerArray;
    $(window).one("hashchange", function() {
        var listenerArray = wok.notificationListeners[msg];
        var del_index = listenerArray.indexOf(func);
        listenerArray.splice(del_index, 1);
        wok.notificationListeners[msg] = listenerArray;
    });
};

wok.notificationsWebSocket = undefined;
wok.startNotificationWebSocket = function () {
    var addr = window.location.hostname + ':' + window.location.port;
    var token = wok.urlSafeB64Encode('woknotifications').replace(/=*$/g, "");
    var url = 'wss://' + addr + '/websockify?token=' + token;
    wok.notificationsWebSocket = new WebSocket(url, ['base64']);

    wok.notificationsWebSocket.onmessage = function(event) {
        var buffer_rcv = window.atob(event.data);
        var messages = buffer_rcv.split("//EOM//");
        for (var i = 0; i < messages.length; i++) {
            if (messages[i] === "") {
                continue;
	    }
            var listenerArray = wok.notificationListeners[messages[i]];
            if (listenerArray == undefined) {
                continue;
            }
            for (var j = 0; j < listenerArray.length; j++) {
                listenerArray[j](messages[i]);
            }
        }
    };

    var heartbeat = setInterval(function() {
        wok.notificationsWebSocket.send(window.btoa('heartbeat'));
    }, 30000);

    wok.notificationsWebSocket.onclose = function() {
        clearInterval(heartbeat);
    };
};


wok.main = function() {
    wok.isLoggingOut = false;
    wok.popable();

    var genTabs = function(tabs) {
        var functionalTabs = [];
        var tabsHtml = [];
        $(tabs).each(function(i, tab) {
            tab_i18n = i18n[tab] ? i18n[tab] : tab;
            var functionality = tab['functionality'];
            var title = tab['title'];
            var path = tab['path'];
            var helpPath = wok.checkHelpFile(path);
            var disableHelp = (helpPath.length == 0 ? "disableHelp" : helpPath);
            tabsHtml.push(
                '<li class="', functionality.toLowerCase() + 'Tab', '">',
                    '<a class="item ', disableHelp, '" href="', path, '">',
                        title,
                    '</a>',
                    '<input name="funcTab" class="sr-only" value="' + functionality.toLowerCase() + '" type="hidden"/>',
                    '<input name="helpPath" class="sr-only" value="' + helpPath + '" type="hidden"/>',
                    '<input name="colorTab1" class="sr-only" value="' + tab['colorTab1'] + '" type="hidden"/>',
                    '<input name="colorTab2" class="sr-only" value="' + tab['colorTab2'] + '" type="hidden"/>',
                '</li>'
            );

            if (functionalTabs.indexOf(functionality) == -1) {
                functionalTabs.push(functionality)
            }
        });

        $('#functionalTabPanel ul').append(genFuncTabs(functionalTabs));
        $('#tabPanel ul').append(tabsHtml.join(''));
        return;
    };

    var genFuncTabs  = function(tabs){
        var functionalTabHtml = [];
        $(tabs).each(function(i, tab_i18n) {
            functionalTabHtml.push(
                '<li>',
                    '<a class="item',' ',tab_i18n.toLowerCase(),'Tab','" href="#">',
                        tab_i18n,
                    '</a>',
                '</li>'
            );
        });
        return functionalTabHtml.join('');
    };

    var parseTabs = function(plugin, xmlData) {
        var tabs = [];
        var funcNode = $(xmlData).find('functionality');
        var functionality = funcNode.text().split(' ').join('\xa0');
        var colorTab1 = funcNode.attr('colorTab1');
        var colorTab2 = funcNode.attr('colorTab2');
        wok.pluginsColor[plugin] = colorTab2;
        $(xmlData).find('tab').each(function() {
            var tab = $(this);
            var titleKey = tab.find('title').text();
            var title = i18n[titleKey] ? i18n[titleKey] : titleKey;
            var path = tab.find('path').text();
            var user_role = wok.cookie.get('user_role');
            var order = tab.find('order').text();

            if (user_role) {
                var mode = tab.find('[role=' + user_role + ']').attr('mode');
                wok.tabMode[titleKey.toLowerCase()] = mode;
                if (mode != 'none') {
                    tabs.push({
                        functionality: functionality,
                        title: title,
                        path: path,
                        mode: mode,
                        order: order,
                        colorTab1: colorTab1,
                        colorTab2: colorTab2
                    });
                }
            } else {
                document.location.href = 'login.html';
            }
        });

        return tabs;
    };

    var retrieveTabs = function(plugin, url) {
        var tabs = [];
        $.ajax({
            url : url,
            async : false,
            context: plugin,
            success : function(xmlData) {
                tabs = parseTabs(this, xmlData);
            },
            statusCode : {
                404: function() {
                    return tabs;
                }
            }
        });
        return tabs;
    };

    var wokConfigUrl = 'ui/config/tab-ext.xml';
    var pluginConfigUrl = 'plugins/{plugin}/ui/config/tab-ext.xml';
    var pluginI18nUrl = 'plugins/{plugin}/i18n.json';
    var DEFAULT_HASH;
    var buildTabs = function(callback) {
        // Make wok.plugins is ready to be used
        if (wok.plugins == undefined) {
            setTimeout(function() {buildTabs(callback)}, 2000);
            return;
        }

        var tabs = retrieveTabs('wok', wokConfigUrl);
        var plugins = wok.plugins;
        $(plugins).each(function(i, p) {
            if (p.enabled === false) {
                return true;
            }

            var url = wok.substitute(pluginConfigUrl, {
                plugin: p.name
            });
            var i18nUrl = wok.substitute(pluginI18nUrl, {
                plugin: p.name
            });
            wok.getI18n(function(i18nObj){ $.extend(i18n, i18nObj)},
                        function(i18nObj){ //i18n is not define by plugin
                        }, i18nUrl, true);
            var pluginTabs = retrieveTabs(p.name, url);
            if(pluginTabs.length > 0){
                tabs.push.apply(tabs, pluginTabs);
            }
        });

        //sort second level tab based on their ordering number
        var orderedTabs = tabs.slice(0);
        orderedTabs.sort(function(a, b) {
            return a.order - b.order;
        });
        //redirect to empty page when no plugin installed
        if(tabs.length===0){
            DEFAULT_HASH = 'wok-empty';
        } else {
            var defaultTab = orderedTabs[0]
            var defaultTabPath = defaultTab && defaultTab['path']

            // Remove file extension from 'defaultTabPath'
            DEFAULT_HASH = defaultTabPath &&
                defaultTabPath.substring(0, defaultTabPath.lastIndexOf('.'))
        }

        genTabs(orderedTabs);
        wok.getHostname();
        wok.logos('ul#plugins',true);
        wok.logos('ul#wok-about',false);

        callback && callback();
    }

    var onLanguageChanged = function(lang) {
        wok.lang.set(lang);
        location.reload();
    };

    /**
     * Do the following setup:
     *   1) Clear any timing events.
     *   2) If the given URL is invalid (i.e., no corresponding href value in
     *      page tab list.), then clear location.href and inform the user;
     *
     *      Or else:
     *      Move the page tab indicator to the right position;
     *      Load the page content via Ajax.
     */
    var onWokRedirect = function(url) {

        if (url == 'wok-empty.html') {
            var warning_msg = "Unable to access WoK User Activity Log feature as a non-root user.<br>No plugins installed currently. You can download the available plugins <a href='https://github.com/kimchi-project/kimchi'>Kimchi</a> and <a href='https://github.com/kimchi-project/ginger'>Ginger</a> from Github."
            $('#main').html(warning_msg).addClass('noPluginMessage');
            return;
        }

        /*
         * Find the corresponding tab node and animate the arrow indicator to
         * point to the tab. If nothing found, inform user the URL is invalid
         * and clear location.hash to jump to home page.
         */
        var tab = $('#tabPanel a[href="' + url + '"]');
        if (tab.length === 0) {
            location.hash = '#' + $('#tabPanel a').attr('href');
            var lastIndex = location.hash.lastIndexOf(".html");
            if (lastIndex != -1) {
                location.hash = location.hash.substring(0, lastIndex);
            }
            return;
        }

        var plugin = $(tab).parent().find("input[name='funcTab']").val();
        var colorTab1 = $(tab).parent().find("input[name='colorTab1']").val();
        var colorTab2 = $(tab).parent().find("input[name='colorTab2']").val();
        var toolbar = $('#toolbar').closest('.navbar-default.toolbar');
        $('#toolbar ul.tools').html('');

        $('#tabPanel').css('background-color', colorTab1);
        $('#tabPanel ul li').removeClass('active');
        $('#tabPanel ul li a').removeAttr('style');
        $.each($('#tabPanel li'), function(i, t) {
            if ($(t).hasClass(plugin + 'Tab')) {
                $(t).css('display', 'block');
            } else {
                $(t).css('display', 'none');
            }
        });

        $(tab).parent().addClass('active');
        $(tab).css('background-color', colorTab2).focus();
        $(toolbar).css('background-color', colorTab2);

        $('#functionalTabPanel ul li').removeClass('active');
        $('#functionalTabPanel ul li').removeAttr('style');
        $('#functionalTabPanel ul .' + plugin + 'Tab').parent().addClass('active').focus();
        $('#functionalTabPanel ul .' + plugin + 'Tab').parent().css('background-color', colorTab1);

        // Disable Help button according to selected tab
        if ($(tab).hasClass("disableHelp")) {
            $('#btn-help').css('cursor', "not-allowed");
            $('#btn-help').off("click");
        }
        else {
            $('#btn-help').css('cursor', "pointer");
            $('#btn-help').on("click", wok.openHelp);
        }
        // Load page content.
        loadPage(url);
    };

    /**
     * Use Ajax to dynamically load a page without a page refreshing. Handle
     * arrow cursor animation, DOM node focus, and page content rendering.
     */
    var loadPage = function(url) {
        // Get the page content through Ajax and render it.
        url && $('#main').load(url, function(responseText, textStatus, jqXHR) {
            if (jqXHR['status'] === 401 || jqXHR['status'] === 303) {
                var isSessionTimeout = jqXHR['responseText'].indexOf("sessionTimeout")!=-1;
                document.location.href = isSessionTimeout ? 'login.html?error=sessionTimeout' : 'login.html';
                return;
            }
        });
    };

    /*
     * Update page content.
     * 1) If user types in the main page URL without hash, then we apply the
     *    default hash. e.g., http://kimchi.company.com:8000;
     * 2) If user types a URL with hash, then we publish an "redirect" event
     *    to load the page, e.g., http://kimchi.company.com:8000/#templates.
     */
    var updatePage = function() {
        // Parse hash string.
        var hashString = (location.hash && location.hash.substr(1));

        /*
         * If hash string is empty, then apply the default one;
         * or else, publish an "redirect" event to load the page.
         */
        if (!hashString) {
            location.hash = DEFAULT_HASH;
        }
        else {
            wok.topic('redirect').publish(hashString + '.html');
        }
    };

    /**
     * Register listeners including:
     * 1) wok redirect event
     * 2) hashchange event
     * 3) Tab list click event
     * 4) Log-out button click event
     * 5) About button click event
     * 6) Help button click event
     * 7) Start notifications loop
     */
    var initListeners = function() {
        wok.topic('languageChanged').subscribe(onLanguageChanged);
        wok.topic('redirect').subscribe(onWokRedirect);

        /*
         * If hash value is changed, then we know the user is intended to load
         * another page.
         */
        window.onhashchange = updatePage;

        /*
         * Register click listener of tabs. Replace the default reloading page
         * behavior of <a> with Ajax loading.
         */
        $('#tabPanel ul').on('click', 'a.item', function(event) {
            var href = $(this).attr('href');
            // Remove file extension from 'href'
            location.hash = href.substring(0,href.lastIndexOf('.'))
            /*
             * We use the HTML file name for hash, like: guests for guests.html
             * and templates for templates.html.
             *     Retrieve hash value from the given URL and update location's
             * hash part. It has 2 effects: one is to publish Wok "redirect"
             * event to trigger listener, the other is to put an entry into the
             * browser's address history to make pages be bookmark-able.
             */
            // Prevent <a> causing browser redirecting to other page.
            event.preventDefault();
        });

        /*
         * Register click listener of second level tabs. Replace the default reloading page
         * behavior of <a> with Ajax loading.
         */
         $('#functionalTabPanel ul li').on('click', 'a.item', function(event) {
            var plugin = $(this).text().toLowerCase();
            var previousPlugin = $('#functionalTabPanel ul li.active a').text().toLowerCase();

            $('#tabPanel').switchClass(previousPlugin + 'Tab', plugin + 'Tab');
            $('#tabPanel ul li').removeClass('active');
            $.each($('#tabPanel li'), function(i, t) {
                if ($(t).hasClass(plugin + 'Tab')) {
                    $(t).css('display', 'block');
                } else {
                    $(t).css('display', 'none');
                }
            });

            $('#functionalTabPanel ul li').removeClass('active');
            $(this).parent().addClass('active').focus();

            var firstTab = $('#tabPanel ul.navbar-nav li.' + plugin + 'Tab').first();
            $(firstTab).addClass('active');

            var href = $('a.item', firstTab).attr('href');
            location.hash = href.substring(0,href.lastIndexOf('.'));
            event.preventDefault();
        });

        // Perform logging out via Ajax request.
        $('#btn-logout').on('click', function() {
            wok.logout(function() {
                wok.isLoggingOut = true;
                document.location.href = "login.html";
            }, function(err) {
                wok.message.error(err.responseJSON.reason);
            });
        });

        // Set handler for about button
        $('#btn-about').on('click', function(event) {
            event.preventDefault();
        });

        $("#aboutModal").append($("#about-tmpl").html());

        // Set handler for help button
        $('#btn-help').on('click', wok.openHelp);

        // start WebSocket
        wok.startNotificationWebSocket();
    };

    var initUI = function() {
        var errorMsg = "";
        $(document).bind('ajaxError', function(event, jqXHR, ajaxSettings, errorThrown) {
            if (!ajaxSettings['wok']) {
                return;
            }

            if (jqXHR['status'] === 401) {
                var isSessionTimeout = jqXHR['responseText'].indexOf("sessionTimeout")!=-1;
                wok.user.showUser(false);
                wok.previousAjax = ajaxSettings;
                $(".empty-when-logged-off").empty();
                $(".remove-when-logged-off").remove();
                document.location.href= isSessionTimeout ? 'login.html?error=sessionTimeout' : 'login.html';
                return;
            }
            else if((jqXHR['status'] == 0) && ("error"==jqXHR.statusText) && !wok.isLoggingOut && errorMsg == "") {
               errorMsg = i18n['WOKAPI6007E'].replace("%1", jqXHR.state());
               wok.message.error(errorMsg);
            }
            if(ajaxSettings['originalError']) {
                ajaxSettings['originalError'](jqXHR, jqXHR.statusText, errorThrown);
            }
        });

        wok.user.showUser(true);
        initListeners();
        updatePage();

        // Overriding Bootstrap Modal windows to allow a stack of modal windows and backdrops
        $(document).on({
            'show.bs.modal': function () {
                var zIndex = 1040 + (10 * $('.modal:visible').length);
                $(this).css('z-index', zIndex);
                setTimeout(function() {
                    $('.modal-backdrop').not('.modal-stack').css('z-index', zIndex - 1).addClass('modal-stack');
                }, 0);
            },
            'hidden.bs.modal': function() {
                if ($('.modal:visible').length > 0) {
                    // restore the modal-open class to the body element, so that scrolling works
                    // properly after de-stacking a modal.
                    setTimeout(function() {
                        $(document.body).addClass('modal-open');
                    }, 0);
                }
            }
        }, '.modal');


    };

    // Load i18n translation strings first and then render the page.
    wok.getI18n(
        function(i18nStrings){ //success
            i18n = i18nStrings;
            buildTabs(initUI);
        },
        function(data){ //error
            wok.message.error(data.responseJSON.reason);
        }
    );

    setTimeout(wok.notificationsLoop, wok.NOTIFICATION_INTERVAL);
};


wok.checkHelpFile = function(path) {
    var lang = wok.lang.get();
    var url = path.replace("tabs", "help/" + lang);
    // Checking if help page exist.
    $.ajax({
        url: url,
        async: false,
        error: function() { url = ""; },
        success: function() { }
    });
    return url;
};

wok.getHostname = function(e) {
    host = window.location.hostname;
    $('span.host-location').text(host);
    return host;
}

wok.openHelp = function(e) {
    var tab = $('#tabPanel ul li.active');
    var url = $(tab).find("input[name='helpPath']").val();
    window.open(url, "Wok Help");
    e.preventDefault();
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

wok.message = function(msg, level, node, closeable, onclick, alertId) {
    "use strict";
    var close = closeable || true;
    var container = node || $('#alert-fields');
    if($("#" + alertId).length === 0) {
        if ($(container).size() < 1) {
            container = $('<div id="alert-fields"/>').appendTo($('#alert-container'));
        }
        var message = '<div '+( alertId ? 'id="'+alertId+'"' : '')+' role="alert" class="alert ' + (level || '') + ' '+( close ? 'alert-dismissible' : '')+' fade in" style="display: none;">';
        if(!node || close) {
            message += '<button type="button" class="close" data-dismiss="alert" aria-label="Close" onclick="' + (onclick || '') + '"><span aria-hidden="true"><i class="fa fa-times-circle"></i></span></button>';
        }
        message += msg;
        message += '</div>';
        var $message = $(message);
        $(container).show();
        $(container).append($message);
        $message.alert();
        $message.fadeIn(100);

        if(!close){
            var timeout = setTimeout(function() {
                $message.delay(4000).fadeOut(100, function() {
                    $message.alert('close');
                    $(this).remove();
                    if ($(container).children().length < 1) {
                        $(container).hide();
                    }
                });
            }, 10000);
        }
    }
};

wok.message.warn = function(msg, node, closeable, alertId) {
    "use strict";
    wok.message(msg, 'alert-warning', node, closeable, null, alertId);
};
wok.message.error = function(msg, node, closeable, alertId) {
    "use strict";
    wok.message(msg, 'alert-danger', node, closeable, null, alertId);
};
wok.message.error.code = function(code, node, closeable, alertId) {
    "use strict";
    var msg = code + ": " + i18n[code];
    wok.message(msg, 'alert-danger', node, closeable, null, alertId);
};
wok.message.success = function(msg, node, closeable, alertId) {
    "use strict";
    wok.message(msg, 'alert-success', node, closeable, null, alertId);
};
wok.message.notify = function(notification, node) {
    "use strict";
    wok.message(notification.message, 'alert-warning', node, true, "wok.removeNotification('" + notification.code + "')");
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

Object.defineProperty(Object.prototype, "getDeepValue", {
    value: function(key) {
        var result=undefined;
        try {
            if(!Array.isArray(key)) {
                key=key.parseKey();
            }
            if(key.length!=0) {
                var tmpName=key.shift();
                if(this[tmpName]!=undefined) {
                    result=this[tmpName];
                }
                if(key.length!=0) {
                    result=result.getDeepValue(key);
                }
            }
        }
        catch (err) {
            //do nothing
        }
        return(result);
        }
});

Object.defineProperty(Object.prototype, "setDeepValue", {
    value: function(key, val) {
        var keys;
        if(Array.isArray(key)) {
            keys=key;
        }
        else {
            keys=key.parseKey();
        }
        if(keys.length!=0) {
            var key=keys.shift();
            if(keys.length==0) {
                if(this[key]==undefined) {
                    this[key]=val;
                }
                else if(Array.isArray(this[key])){
                    this[key].push(val);
                }
                else {
                    var tmpArray=[]
                    tmpArray.push(this[key]);
                    tmpArray.push(val);
                    this[key]=tmpArray;
                }
            }
            else {
                if(this[key]==undefined) {
                    this[key]=new Object();
                    this[key].setDeepValue(keys,val);
                }
                else if(Array.isArray(this[key])){
                    var tmpO=new Object();
                    this[key].push(tmpO);
                    tmpO.setDeepValue(keys,val);
                }
                else {
                   this[key].setDeepValue(keys,val);
                }
            }
        }
        return(this);
    }
});
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.popable = function() {
    $(document).click(function(e) {
        $('.popable').each(function(i, n) {
            n === e.target || $.contains(n, e.target) ||
                $('.popover', n).toggle(false);
        });
     });
     $(document).on('click', '.popable', function(e) {
        var popup = $('.popover', this)[0];
        $(popup).toggle();

        // Scroll the popup menu into viewport if invisible.
        !$(popup).is(':visible') ||
            wok.isElementInViewport(popup) ||
            popup.scrollIntoView();
    });
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

wok.select = function(id, options) {
    var listControl = $('#'+ id);
    var targetId = listControl.data('target');
    var labelId = listControl.data('label');
    var value = $('#' + targetId).val();
    var item;
    var itemTag = 'li';
    var selectedClass = 'active';
    $.each(options, function(index, option) {
        item = $('<' + itemTag + '></' + itemTag + '>');
        item.text(option.label);
        item.data('value', option.value);
        if(option.value === value) {
            item.addClass(selectedClass);
            $('#' + labelId).text(option.label);
        }
        listControl.append(item);
    });

    listControl.on('click', itemTag, function() {
        listControl.children().removeClass(selectedClass);
        $(this).addClass(selectedClass);
        $('#' + labelId).text($(this).text());
        var target = $('#' + targetId);
        var oldValue = target.val();
        var newValue = $(this).data('value');
        target.val(newValue);
        if(oldValue !== newValue) {
            target.change();
        }
    });
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.session = {

	remaingTimeToShowAlert: 30000,
	remainingSessionTime: null,
	flagInTimer: false,
	expiringTimeout: null,
	oneSecondExternalCounter: null,
	oneSecondInternalCounter: null,

	expiringCounter: function(){
		var counter = wok.session.remainingSessionTime - wok.session.remaingTimeToShowAlert;
		if (!isNaN(counter)) {
			wok.session.expiringTimeout = setTimeout(function(){
				wok.session.flagInTimer = true;
				$("#session-expiring-alert").show();
				$("#session-expiring-alert p").html("<script>var message = i18n['WOKSESS0001M'].replace('%1', 30);"
					+ "$('#session-expiring-alert p').html(message);"
					+ "var n = 30;"
					+ "wok.session.oneSecondExternalCounter = setTimeout(countDown,1000);"
					+ "function countDown(){n--; if(n > 0){wok.session.oneSecondInternalCounter = setTimeout(countDown,1000);}"
					+ "message = i18n['WOKSESS0001M'].replace('%1', n);"
					+ "$('#session-expiring-alert p').html(message);}</script>"
				);
			}, counter);
		}
	},

	refreshExpiringCounter: function() {
		clearTimeout(wok.session.oneSecondExternalCounter);
		clearTimeout(wok.session.oneSecondInternalCounter);
		clearTimeout(wok.session.expiringTimeout);
		wok.session.expiringTimeout = null;
		wok.session.oneSecondExternalCounter = null;
		wok.session.oneSecondInternalCounter = null;
	},

	renewSession: function(){
		wok.getTasks(function(){}, function(){});
		wok.session.flagInTimer = false;
		clearTimeout(wok.session.oneSecondExternalCounter);
		clearTimeout(wok.session.oneSecondInternalCounter);
		wok.session.oneSecondExternalCounter = null;
		wok.session.oneSecondInternalCounter = null;
		$("#session-expiring-alert").hide();
	},

	hideExpiringAlert: function(){
		$("#session-expiring-alert").hide();
	}
};/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

Object.defineProperty(String.prototype, "parseKey", {
    value: function(parsedKey) {
        try {
            if (!Array.isArray(parsedKey)) {
                parsedKey=[];
            }
        }
        catch (err) {
            parsedKey=[];
        }
        var openBracket=this.indexOf("[");
        if (openBracket!=-1) {
            var id=this.slice(0, openBracket);
            parsedKey.push(id);
            var closeBracket=this.lastIndexOf("]");
            if (closeBracket==-1) {
                closeBracket=this.length;
            }
            var tmpName=this.slice(openBracket+1,closeBracket);
            tmpName.parseKey(parsedKey);
        }
        else {
            parsedKey.push(this);
        }
        return(parsedKey);
        }
});
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.substitute = function(templateStr, data, tag) {
    tag = tag || /\{([^\}]+)\}/g;

    var escapeHtml = function(html) {
        return String(html)
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;');
    };

    return templateStr.replace(tag, function(matchResult, express) {
        var propertyArray = express.split('!');
        var defaultValue = propertyArray[1] || '';
        propertyArray = propertyArray[0].split('.');
        var value = data, i = 0, l = propertyArray.length, property;
        for (; i < l; i++) {
            property = propertyArray[i];
            if (value) {
                value = value[property];
            } else {
                break;
            }
        }
        return escapeHtml((value || value === 0) ? value : defaultValue);
    });
};
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * pub/sub
 * Usage:
 *   Publish - wok.topic('eventname').publish(params);
 *   Subscribe - wok.topic('eventname').subscribe(listener);
 *   Unsubscribe - wok.topic('eventname').unsubscribe(listener);
 */
wok.topic = (function() {

    var topics = {};

    return function( id ) {
        var callbacks,
            method,
            topic = id && topics[ id ];

        if ( !topic ) {
            callbacks = jQuery.Callbacks();
            topic = {
                publish: callbacks.fire,
                subscribe: callbacks.add,
                unsubscribe: callbacks.remove
            };
            if ( id ) {
                topics[ id ] = topic;
            }
        }
        return topic;
    };
})();
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.user = (function() {
    var getUserName = function() {
        return wok.cookie.get('username');
    };

    var setUserName = function(userName) {
        wok.cookie.set('username', userName, 365);
    };

    var showUser = function(toShow) {
        if (toShow) {
            var userName = getUserName();
            userName && $('#user-name').text(userName);
            $('#user-login').removeClass('not-logged-in');
            return;
        }

        $('#user-login').addClass('not-logged-in');
    };

    return {
        getUserName: getUserName,
        setUserName: setUserName,
        showUser: showUser
    };
})();
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * To check whether a given DOM node is entirely within the viewport
 * of a browser.
 * @param {DOMObject} el The given DOM node to check.
 *
 * @returns {true|false|undefined}
 *     true if the node is within viewport, or
 *     false if the node is not entirely visible, or
 *     undefined if the given parameter is invalid.
 */
wok.isElementInViewport = function(el) {
    if (!el || !el.getBoundingClientRect) {
        return undefined;
    }

    var rect = el.getBoundingClientRect();

    return (
        rect.top >= 0 &&
        rect.left >= 0 &&
        rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
        rect.right <= (window.innerWidth || document.documentElement.clientWidth)
    );
};

/**
 * To change the byte to proper unit.
 * @param number needed to change unit.
 * @param digits after the decimal point.
 * @returns str with unit.
 */
wok.changetoProperUnit = function(numOrg, digits, base) {
    if (numOrg === undefined) {
        return "";
    }
    var suffixes = [ 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'B' ];
    var base = base || 1024;
    var numTemp = numOrg;
    var result = numOrg.toFixed(digits) + 'B';
    for ( var i = 0; i < suffixes.length; i++) {
        var numTemp = numTemp / base;
        if (numTemp < 1)
            break;
        result = numTemp.toFixed(digits) + suffixes[i]
    }
    return result;
};

/**
 * wok.formatMeasurement util.
 *
 * Refer to "Units of information" (
 *   http://en.wikipedia.org/wiki/Units_of_information
 * ) for more information about measurement units.
 *
 * @param number The number to be normalized.
 * @param settings
 *     * base Measurement base, accepts 2 or 10. defaults to 2.
 *     * unit The unit of the measurement, e.g., B, Bytes/s, bps, etc.
 *     * fixed The number of digits after the decimal point.
 *
 * @returns [object]
 *     * v The number part of the measurement.
 *     * s The suffix part of the measurement including multiple and unit.
 *         e.g., kB/s means 1000B/s, KiB/s for 1024B/s.
 */
(function() {
    var unitBaseMapping = {
        2: [{
            us: 'Ki',
            v: 1024
        }, {
            us: 'Mi',
            v: 1048576
        }, {
            us: 'Gi',
            v: 1073741824
        }, {
            us: 'Ti',
            v: 1099511627776
        }, {
            us: 'Pi',
            v: 1125899906842624
        }],
        10: [{
            us: 'k',
            v: 1000
        }, {
            us: 'M',
            v: 1000000
        }, {
            us: 'G',
            v: 1000000000
        }, {
            us: 'T',
            v: 1000000000000
        }, {
            us: 'P',
            v: 1000000000000000
        }]
    };

    var Formatted = function(value, suffix) {
        this['v'] = value;
        this['s'] = suffix;
    };
    Formatted.prototype.toString = function() {
        return this['v'] + this['s'];
    };

    var format = function(number, settings) {
        if(number === (undefined || null) || isNaN(number)) {
            return number;
        }

        var settings = settings || {};
        var unit = settings['unit'] || 'B';
        var base = settings['base'] || 2;
        if(base !== 2 && base !== 10) {
            return new Formatted(number, unit);
        }

        // Introduce converter to format data.
        // Converter's to function will be used for formatting.
        // if not passed then 'to' formatting will not be done.
        // e.g. formatMeasurement( 2,
        //          { .. , converter: wok.localeConverters["number-locale-converter"]})
        var converter = settings['converter'] || null;
        var converter_to_fun = converter?converter['to']:''
        var fixed = settings['fixed'];

        var unitMapping = unitBaseMapping[base];
        var unitmap = { 'Ki': 'WOKFMT2001M', 'Mi': 'WOKFMT2002M', 'Gi': 'WOKFMT2003M',
                        'Ti': 'WOKFMT2004M', 'Pi': 'WOKFMT2005M', 'k': 'WOKFMT2006M',
                        'M': 'WOKFMT2007M', 'G': 'WOKFMT2008M', 'T': 'WOKFMT2009M',
                        'P': 'WOKFMT2010M'}
        for(var i = unitMapping.length - 1; i >= 0; i--) {
            var mapping = unitMapping[i];
            var s_key = mapping['us'];
            var suffix = i18n[unitmap[s_key]];
            var startingValue = mapping['v'];
            if(number < startingValue) {
                continue;
            }

            var formatted = number / startingValue;
            formatted = fixed ? formatted.toFixed(fixed) : formatted;
            formatted = converter_to_fun ? converter_to_fun(Number(formatted)) : formatted;
            return new Formatted(formatted, suffix + unit);
        }

        formatted = fixed ? number.toFixed(fixed) : number;
        /* format the formatted number as per settings's converter, if not present return as it is. */
        formatted_val = converter_to_fun ? converter_to_fun(Number(formatted)) : formatted;
        return new Formatted(formatted_val, unit);
    };

    wok.formatMeasurement = format;
})();

wok.isUnsignedNumeric = function(number) {
    var reg = /^d+(.d+)?$/
    return reg.test(number);
}

wok.isServer = function(server) {
    var domain = "([0-9a-z_!~*'()-]+\.)*([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\.[a-z]{2,6}";
    var ip = "(\\d{1,3}\.){3}\\d{1,3}";
    regex = new RegExp('^' + domain + '|' + ip + '$');
    if (!regex.test(server)) {
        return false;
    } else {
        return true;
    }
};

wok.escapeStr = function(str) {
    if (str)
        return str.replace(/([ #;?%&,.+*~\\':"!^$[\]()<=>`{|}\/@])/g,'\\$&');

    return str;
};

wok.urlSafeB64Decode = function(str) {
    return $.base64.atob(str.replace(/-/g, '+').replace(/_/g, '/'), true);
}

wok.urlSafeB64Encode = function(str) {
    return $.base64.btoa(str, true).replace(/\+/g, '-').replace(/\//g, '_');
}

wok.notificationsLoop = function notificationsLoop() {
    wok.getNotifications(
        function(notifications){
            if(notifications && notifications.length > 0) {
                $.each(notifications, function(i, notif) {
                    // Check if notification is being displayed
                    if (!$("#notification-container").length) {
                        $('.navbar.toolbar').next().prepend('<div id="notification-container"></div>');
                    }
                    if (($("#notification-container").find("div:contains('" + notif.message + "')").length) == 0) {
                        wok.message.notify(notif, '#notification-container');
                    }
                });
            }
            setTimeout(notificationsLoop, wok.NOTIFICATION_INTERVAL);
        },
        function(data){
            setTimeout(notificationsLoop, wok.NOTIFICATION_INTERVAL);
        }
    );
}

wok.datetimeLocaleConverter = function datetimeLocaleConverter(datetime_string, locale){
   var dateRegEx = /(\d{4})-(\d{2})-(\d{2})/;
   if(dateRegEx.test(datetime_string.substr(0,10))){
     var dte = new Date(datetime_string.substr(0,10) + 'T' + datetime_string.substr(11));
     var options = { year: 'numeric', month: 'long', day: 'numeric' };
     return dte.toLocaleString(locale, options);
  }else{
    return datetime_string;
  }
}

wok.dateLocaleConverter = function dateLocaleConverter(date_string, locale){
     var dte = new moment(date_string);
     var options = { year: 'numeric', month: 'numeric', day: 'numeric' };
     return dte._d.toLocaleDateString(locale, options);
}

wok.timeLocaleConverter = function timeLocaleConverter(time_string, locale){
     var dte = new Date((new Date(0)).toDateString() + ' ' + time_string);
     return dte.toLocaleTimeString(locale);
}

wok.numberLocaleConverter = function numberConverter(number, locale){
     number = (typeof(number) === 'number') ? number.toLocaleString(wok.lang.get_locale()) : number;
     return number;
}

wok.localeConverters = {
       "date-locale-converter": {
           to: function(date){
              return wok.dateLocaleConverter(date, wok.lang.get_locale());
           }
       },
       "time-locale-converter": {
           to: function(time){
              return wok.timeLocaleConverter(time, wok.lang.get_locale());
           }
       },
       "datetime-locale-converter": {
            to: function(datetime){
                return wok.datetimeLocaleConverter(datetime, wok.lang.get_locale());
            }
       },
       "number-locale-converter":{
           to: function(number){
              if (number == null) {
                 return 'Unknown';
              }
              format_value = wok.numberLocaleConverter(number, wok.lang.get_locale());
              return format_value.toString().replace(/\s/g,' '); //replace non-breaking space with breaking space
           }
      }
}
/*
 * Project Wok
 *
 * Copyright IBM Corp, 2015-2016
 *
 * Code derived from Project Kimchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
wok.window = (function() {
    "use strict";
    var _windows = [];
    var _listeners = {};
    var open = function(settings, target) {
        settings = jQuery.type(settings) === 'object' ? settings : {
            url: settings
        };

        target = (typeof target === 'undefined') ? 'modalWindow' : target;

        var windowID = settings['id'] || 'window-' + _windows.length;

        if ($('#' + windowID).length) {
            $('#' + windowID).remove();
        }

        _windows.push(windowID);
        _listeners[windowID] = settings['close'];
        var windowNode = $('<div id="' + windowID + '" class="modal-dialog"></div>');

        $('#' + target).modal({
            backdrop: 'static',
            keyboard: false
        });

        $('#' + target).modal('show');

        $('#' + target).on('hidden.bs.modal', function() {
            $('#' + windowID).remove();
        });

        $(windowNode).appendTo('#' + target).on('click', '.modal-header > .close', function() {
            wok.window.close();
        });

        if (settings['url']) {
            $(windowNode).load(settings['url']).fadeIn(100);
            return;
        }

        settings['content'] && $(windowNode).html(settings['content']);
    };

    var close = function() {
        var windowID = _windows.pop();
        if (_listeners[windowID]) {
            _listeners[windowID]();
            _listeners[windowID] = null;
        }
        $('#' + windowID).parent().modal('toggle');
        $('#' + windowID).parent().on('hidden.bs.modal', function() {
            delete _listeners[windowID];
            $('#' + windowID).remove();
        })

    };

    return {
        open: open,
        close: close
    };
})();
